@@ -1,11 +1,12 @@
 /* App.m
  * Application class of Cenon
  *
- * Copyright (C) 1995-2010 by vhf interservice GmbH
+ * Copyright (C) 1995-2011 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1995-08-10
- * modified: 2010-07-04 (svg stuff added)
+ * modified: 2011-03-06 (-applicationDidFinishLunching: auto check for updates)
+ *           2010-07-04 (svg stuff added)
  *           2010-06-30 (-changeSafeType: TIFF added)
  *           2010-04-10 (displayInfo: get version from plist and date from __DATE__)
  *           2010-01-12 (take snapshot of open documents and their positions)
@@ -274,7 +275,7 @@
 }
 
 /* created:  1993-01-??
- * modified: 2005-11-18
+ * modified: 2011-03-06
  */
 - (void)applicationDidFinishLaunching:(NSNotification *)notification
 {   NSApplication	*theApplication = [notification object];
@@ -328,6 +329,12 @@
 
     /* Load Document Snapshot */
     [self restoreSnapshot:self];
+
+    /* automatically check for updates */
+    [[UpdateController sharedInstance] checkForUpdates:self];
+    /*[[NSRunLoop currentRunLoop] performSelector:@selector(checkForUpdates:)
+     target:[UpdateController sharedInstance]
+     argument:self order:10 modes:NSDefaultRunLoopMode];*/
 }
 
 
