@@ -5,7 +5,8 @@
  * Author:   Georg Fleischmann
  *
  * created:  1996-09-17
- * modified: 2010-03-03 (-copy setDirectionCCW, -pathRepresentation edges build with r, not radius)
+ * modified: 2010-07-17 (-transform added, -scale: scales width correctly with negative scales (flip))
+ *           2010-03-03 (-copy setDirectionCCW, -pathRepresentation edges build with r, not radius)
  *           2006-02-08
  *
  * This program is free software; you can redistribute it and/or
@@ -522,17 +523,38 @@
     [super setSelected:flag];
 }
 
+- (void)transform:(NSAffineTransform*)matrix
+{   NSSize  s = NSMakeSize(width, width);
+
+    s = [matrix transformSize:s];
+    width = (Abs(s.width) + Abs(s.height)) / 2;
+    origin = [matrix transformPoint:origin];
+    size   = [matrix transformPoint:size];
+    if (size.x < 0.0)
+    {   origin.x += size.x;
+        size.x = -size.x;
+    }
+    if (size.y < 0.0)
+    {   origin.y += size.y;
+        size.y = -size.y;
+    }
+    dirty = graduateDirty = YES;
+}
+
 - (void)scale:(float)x :(float)y withCenter:(NSPoint)cp
 {
-    width *= (x+y)/2.0;
+    width *= (Abs(x)+Abs(y))/2.0;
     origin.x = ScaleValue(origin.x, cp.x, x);
+    if ( x < 0.0)
+        origin.x += size.x * x;
     origin.y = ScaleValue(origin.y, cp.y, y);
-    size.x *= x;
-    size.y *= y;
+    if ( y < 0.0)
+        origin.y += size.y * y;
+    size.x *= Abs(x);
+    size.y *= Abs(y);
     coordBounds = NSZeroRect;
 
-    dirty = YES;
-    graduateDirty = YES;
+    dirty = graduateDirty = YES;
 }
 
 - (void)mirrorAround:(NSPoint)p
@@ -639,6 +661,9 @@
         {   NSColor	*oldColor=nil;
             float	defaultWidth = [NSBezierPath defaultLineWidth];
 
+            if ( filled && (Diff(size.x, 0) < 500*TOLERANCE || Diff(size.y, 0) < 500*TOLERANCE) )
+                defaultWidth = 0.1 / [principal scaleFactor];   // make visible what actually is invisible
+
             if (!VHFIsDrawingToScreen() && [principal separationColor])
             {   NSColor	*sepColor = [self separationColor:color]; // get individual separation color
 
@@ -2319,7 +2344,7 @@
     if (!filled && [plist objectForKey:@"filled"])
         filled = 1;
     if (!(fillColor = colorFromPropertyList([plist objectForKey:@"fillColor"], [self zone])))
-        [self setFillColor:[color copy]];
+        [self setFillColor:[NSColor blackColor]/*[color copy]*/];
     if (!(endColor = colorFromPropertyList([plist objectForKey:@"endColor"], [self zone])))
         [self setEndColor:[NSColor blackColor]];
     graduateAngle = [plist floatForKey:@"graduateAngle"];
