@@ -1,11 +1,15 @@
 /* DXFImport.m
  * DXF import object
  *
- * Copyright (C) 1996-2010 by vhf interservice GmbH
+ * Copyright (C) 1996-2011 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-05-01
- * modified: 2010-04-03 (-getArc: negative 360 deg arcs work now, buildArc(): dx == 0 case)
+ * modified: 2011-02-27 (-init: initialize group.handle, state.color, blockScanner, rable, vivibleList)
+ *           2010-10-31 (buildArc(): dx, dy calced with double)
+ *           2010-09-11 (-interpret: try UTF8 and Latin encodings to make GNUstep happy)
+ *           2010-06-11 (buildArc(): 360.0 - acos(cn), was 180.0 - )
+ *           2010-04-03 (-getArc: negative 360 deg arcs work now, buildArc(): dx == 0 case)
  *           2007-01-30 (close PolyLine: check for start == end point)
  *
  * This file is part of the vhf Import Library.
@@ -59,7 +63,7 @@ static BOOL gotoTable(NSScanner *scanner, NSString *name);
 static BOOL gotoGroup( NSScanner *scanner, NSString *name);
 static NSArray *getTableFromData(NSString *data);
 static NSColor *getColorFromTable(NSString *str, NSArray *table);
-static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *angle);
+static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *angle);
 
 @interface DXFImport(PrivateMethods)
 - (BOOL)getHeader:(NSString*)data;
@@ -413,6 +417,13 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
     group.text = nil;
     group.name = nil;
     group.layer = nil;
+    group.handle = nil;
+
+    state.color = nil;
+
+    blockScanner = nil;
+    table = nil;
+    visibleList = nil;
 
     return self;
 }
@@ -423,12 +434,17 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 /* created:   1996-01-25
- * modified:  2002-10-26
+ * modified:  2010-09-11 (load UTF8 instead of ASCII, if it fails try Latin)
  * parameter: dxfData	the DXF data stream
  * purpose:   start interpretation of dxfData
  */
 - (id)importDXF:(NSData*)dxfData
-{   NSString	*data = [[[NSString alloc] initWithData:dxfData encoding:NSASCIIStringEncoding] autorelease];
+{   NSString	*data = [[[NSString alloc] initWithData:dxfData
+                                            encoding:NSUTF8StringEncoding] autorelease];
+
+    if ( ! data )   // try latin
+        data = [[[NSString alloc] initWithData:dxfData
+                                      encoding:NSISOLatin1StringEncoding] autorelease];
 
     newLineSet = [NSCharacterSet characterSetWithCharactersInString:@"\r\n"];
     skipSet = [NSCharacterSet characterSetWithCharactersInString:@" \t"];
@@ -1011,18 +1027,21 @@ static NSColor *getColorFromTable(NSString *str, NSArray *table)
 }
 
 /* Note: see page 323 of DXF book
- * modified: 2010-04-03
+ * modified: 2010-10-31 (dx, dy calced with double)
+ *           2010-06-11 (360 - acos(cn))
+ *           2010-04-03
  */
-static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *angle)
-{   NSPoint	p, m;
+static void buildArc(NSPoint beg, NSPoint end, double a, NSPoint *ctr, float *angle)
+{   NSPoint	p;
     double	l, ang, dx, dy, r, an, h;
+    double  mx, my, begx = beg.x, begy = beg.y, endx = end.x, endy = end.y;
 
     ang = 4.0 * RadToDeg(atan( a ));
     l = sqrt(SqrDistPoints(beg, end));  // length of arc from point to point
     h = (a*l)/2.0;                      // height of arc
-    m.x = (beg.x+end.x)/2.0; m.y = (beg.y+end.y)/2.0;   // mit point between start and end
-    dx = m.x - beg.x;                   // 
-    dy = m.y - beg.y;                   // 
+    mx = (begx+endx)/2.0; my = (begy+endy)/2.0; // mit point between start and end
+    dx = mx - begx;                     // 
+    dy = my - begy;                     // 
     //if (h>0)	/* don't know if this is correct */
     //    ang = -ang;
 //printf("h:%f ang:%f\n", h, ang);
@@ -1030,7 +1049,7 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
     if ( Abs(dx) < TOLERANCE )              // larger y component (vertical arc) (only tested for vertical case !)
     {   double  cn = Max( -1.0, Min( 1.0, dx / (l/2.0) ) );
         if ( beg.y > end.y )
-            an = 180.0 - (RadToDeg(acos(cn)) - (((ang<0) ? -90.0 : 90.0)-ang/2.0));
+            an = 360.0 - (RadToDeg(acos(cn)) - (((ang<0) ? -90.0 : 90.0)-ang/2.0));
         else
             an = RadToDeg(acos(cn)) + (((ang<0) ? -90.0 : 90.0)-ang/2.0);
     }
@@ -1046,7 +1065,7 @@ static void buildArc(NSPoint beg, NSPoint end, float a, NSPoint *ctr, float *ang
     p.x = beg.x + r; p.y = beg.y;
     vhfRotatePointAroundCenter(&p, beg, an);
     *ctr = p;
-    *angle = ang;
+    *angle = (float)ang;
 }
 
 /* created:   18.05.93
