@@ -1,11 +1,13 @@
 /* vhf2DFunctions.h
  * vhf 2-D graphic functions
  *
- * Copyright (C) 1996-2009 by vhf interservice GmbH
+ * Copyright (C) 1996-2011 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-19
- * modified: 2008-02-17 (SqrPoint() added)
+ * modified: 2011-04-26 (vhfConvertLineToLines(), vhfBoundsOfCurve())
+ *           2010-06-11 (SqrDistPoints(), SqrPoint() casted to double)
+ *           2008-02-17 (SqrPoint() added)
  *           2008-08-18 (vhfIsIntValueInArray() added, valueInArray() is deprecated now)
  *
  * This file is part of the vhf Shared Library.
@@ -39,8 +41,18 @@
 
 #include "types.h"
 
-#define	SqrDistPoints(p1, p2)   (((p1).x-(p2).x)*((p1).x-(p2).x)+((p1).y-(p2).y)*((p1).y-(p2).y))
-#define	SqrPoint(p)             ((p).x*(p).x + (p).y*(p).y)
+//#define	SqrDistPoints(p1, p2)   (((p1).x-(p2).x)*((p1).x-(p2).x)+((p1).y-(p2).y)*((p1).y-(p2).y))
+#define	SqrDistPointsM(p1, p2)  (((double)(p1).x-(double)(p2).x)*((double)(p1).x-(double)(p2).x)+((double)(p1).y-(double)(p2).y)*((double)(p1).y-(double)(p2).y))
+static __inline__ double SqrDistPoints(NSPoint p1, NSPoint p2)
+{   double	x1 = p1.x, y1 = p1.y, x2 = p2.x, y2 = p2.y;
+    return ((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
+}
+//#define	SqrPoint(p)             ((p).x*(p).x + (p).y*(p).y)
+//#define	SqrPoint(p)             ((double)(p).x*(double)(p).x + (double)(p).y*(double)(p).y)
+static __inline__ double SqrPoint(NSPoint p)
+{   double	x = p.x, y = p.y;
+    return x*x + y*y;
+}
 
 /* distance */
 float distancePointLine(const NSPoint *p0, const NSPoint *p1, const NSPoint *point);
@@ -91,6 +103,7 @@ int     removePointWithToleranceFromArray(NSPoint p, float tol, NSPoint *pArray,
 int     vhfFilterPoints(NSPoint *pArray, int cnt, float tol);
 
 /* bezier curve */
+NSRect  vhfBoundsOfCurve( NSPoint pc[4] );
 int     vhfSplitCurveAt(const NSPoint pc[4], float t, NSPoint *pc1, NSPoint *pc2);
 BOOL    vhfGetPointOnCurveAt(NSPoint *p, NSPoint pc[4], float t);
 BOOL    vhfGetGradientOfCurveAt(NSPoint *p, NSPoint pc[4], float t);
@@ -101,4 +114,7 @@ int     vhfConvertCurveToLines(NSPoint pc[4], float flatness, NSPoint **pts);
 NSRect  vhfBoundsOfArc(NSPoint center, float radius, float begAngle, float angle);
 int     vhfConvertArcToLines(NSPoint center, float radius, float begAngle, float angle, float flatness, NSPoint **pts);
 
+/* line */
+int     vhfConvertLineToLines(NSPoint pc[2], float flatness, NSPoint **pts);
+
 #endif // VHF_H_2DFUNCTIONS
