@@ -1,11 +1,13 @@
 /* vhf2DFunctions.m
  * vhf 2-D graphic functions
  *
- * Copyright (C) 1996-2004 by vhf interservice GmbH
+ * Copyright (C) 1996-2011 by vhf interservice GmbH
  * Author:   Georg Fleischmann
  *
  * created:  1996-01-19
- * modified: 2003-08-08
+ * modified: 2011-04-26 (vhfConvertLineToLines(), vhfBoundsOfCurve())
+ *           2008-02-17 (some clean-up)
+ *           2008-08-18 (vhfIsIntValueInArray() added, valueInArray() is deprecated now)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the vhf Public License as
@@ -28,6 +30,7 @@
 
 #include <math.h>
 #include "vhf2DFunctions.h"
+#include "vhfMath.h"
 
 /*
  * created:   04.09.93
@@ -57,6 +60,51 @@ float sqrDistancePointLine(const NSPoint *p0, const NSPoint *p1, const NSPoint *
 
     return pointOnLineClosestToPoint(*p0, *p1, *point, &iPoint);
 }
+float vhfSqrDistancePointLine(NSPoint p0, NSPoint p1, NSPoint point)
+{   NSPoint	iPoint;
+
+    return pointOnLineClosestToPoint(p0, p1, point, &iPoint);
+}
+
+/* modified: 09.11.96
+ */
+float distancePointArc(NSPoint pt, NSPoint cp, float r, float ba, float angle)
+{   float	dist;		// the distance to the full circle
+    float	dx, dy, c;
+    float	an;		// angle of point
+    NSPoint	p0, pa;
+    float	da, db;		// distances to endpoints
+    float	ea = ba + angle;
+
+    /* we need positive angles with ea > ba */
+    if (angle < 0.0)
+    {   float	v; v=ba; ba = ea; ea = v;}
+    if (ba < 0.0) ba += 360.0;
+    if (ba >= 360.0) ba -= 360.0;
+    ea = ba + Abs(angle);
+
+    dx = pt.x - cp.x; dy = pt.y - cp.y;
+    c = sqrt(dx*dx+dy*dy);
+    dist = Diff(r, c);	/* distance to full circle */
+
+    if (Diff(ba, ea) >= 360.0)	/* this is a full arc */
+        return dist;
+
+    /* compare angle of p */
+    an = vhfAngleOfPointRelativeCenter(pt, cp);
+    if (an < ba) an += 360.0;
+    if (an >= ba && an <= ea)	/* we are inside the angles */
+        return dist;
+
+    /* distance to endpoints */
+    p0.x = cp.x + r; p0.y = cp.y;
+    pa = vhfPointAngleFromRefPoint(cp, p0, ba);
+    da = SqrDistPoints(pa, pt);
+    pa = vhfPointAngleFromRefPoint(cp, p0, ea);
+    db = SqrDistPoints(pa, pt);
+
+    return sqrt((da < db) ? da : db);	/* distance to endpoint */
+}
 
 /*
  * created:  1996-03-29
@@ -299,7 +347,7 @@ int vhfIntersectLines(NSPoint *pArray, NSPoint p0, NSPoint p1, NSPoint pl0, NSPo
 }
 
 /* created:   2003-07-19
- * modified:  2003-07-19
+ * modified:  2009-12-27 (init pr1)
  * purpose:   intersect line with a rect
  * parameter: pts[2] (intersections)
  *            p0, p1
@@ -311,7 +359,7 @@ int vhfIntersectLineAndRect(NSPoint p0, NSPoint p1, NSRect rect, NSPoint *pts)
 {   int		i, j, cnt, pCnt = 0;
     NSPoint	pr0, pr1, pArray[2];
 
-    pr0 = rect.origin;
+    pr0 = pr1 = rect.origin;
     for (i=0; i<4; i++)
     {
         switch (i)
@@ -329,6 +377,42 @@ int vhfIntersectLineAndRect(NSPoint p0, NSPoint p1, NSRect rect, NSPoint *pts)
     return pCnt;
 }
 
+/* return intersection between two circles
+ * created: 2009-12-21
+ *
+ * r1*r1 - c1*c1 = r2*r2 - c2*c2, c2 = d-c1
+ * r1: radius 1
+ * r2: radius 2
+ * d:  distance of centers
+ * c:  distance from one center (on d) to the center of the intersections
+ */
+int vhfIntersectCircles(NSPoint cp1, float r1, NSPoint cp2, float r2, NSPoint *pts)
+{   double  distCtr = sqrt(SqrDistPoints(cp1, cp2)); // distance between centers
+    double  tol = TOLERANCE;
+    double  c, at, mdx, mdy, px, py, v;
+
+    /* a quick check for possible intersection */
+    if ( r1+r2+tol < distCtr )
+        return 0;
+
+    c = ((r1*r1) - (r2*r2) + distCtr*distCtr) / (2.0*distCtr);
+    at = ( Diff(r1+r2, distCtr) <= tol || Diff(Diff(r1, r2), distCtr) <= tol )
+    ? 0.0                   // 1 intersection only
+    : sqrt(r1*r1 - c*c);    // 2 intersections
+    mdx = cp2.x - cp1.x;
+    mdy = cp2.y - cp1.y;
+    px = cp1.x + mdx*c/distCtr;
+    py = cp1.y + mdy*c/distCtr;
+
+    v = mdx; mdx = -mdy; mdy = v;
+    pts[0].x = px + mdx*at/distCtr;
+    pts[0].y = py + mdy*at/distCtr;
+    pts[1].x = px - mdx*at/distCtr;
+    pts[1].y = py - mdy*at/distCtr;
+
+    return (!at) ? 1 : 2;
+}
+
 /* the problem of the OpenStep-Funktion is that a width or height of 0 gives no intersection
  */
 BOOL vhfIntersectsRect(NSRect rect1, NSRect rect2)
@@ -366,7 +450,8 @@ BOOL vhfIntersectsRect(NSRect rect1, NSRect rect2)
     return NO;
 }
 
-/* check whether p is inside the poly line defined by nPts points in pts
+
+/* check whether p is inside the polygon defined by nPts points in pts
  */
 BOOL vhfIsPointInsidePolygon(NSPoint p, NSPoint *pts, int nPts)
 {   int		i, cnt, leftCnt = 0;
@@ -437,6 +522,11 @@ BOOL vhfContainsRect(NSRect rect1, NSRect rect2)
     return NSContainsRect(rect1, rect2);
 }
 
+
+/*
+ * Rotation
+ */
+
 /* cw
  */
 NSPoint vhfPointRotatedAroundCenter(NSPoint p, float a, NSPoint cp)
@@ -464,6 +554,79 @@ void vhfRotatePointAroundCenter(NSPoint *p, NSPoint cp, float a)
     p->y = np.y + cp.y;
 }
 
+
+/*
+ * Angle
+ */
+
+/* calculate the angle of p relative cp
+ * right of cp = 0, up = 90, left = 180, down = 270
+ */
+float vhfAngleOfPointRelativeCenter(NSPoint p, NSPoint cp)
+{   float a, dx = p.x - cp.x, dy = p.y - cp.y;
+
+    if (!dx)
+        a = (dy >= 0) ? 90.0 : 270.0;
+    else
+    {	a = RadToDeg(atan(dy / dx));
+        if (dx<0) a += 180;         // 2, 3
+        if (dx>0 && dy<0) a += 360; // 4
+    }
+    return a;
+}
+
+/* calculates a point on the radius with a distance of angle (degree) from a reference point
+ */
+NSPoint vhfPointAngleFromRefPoint(NSPoint cp, NSPoint refP, float angle)
+{
+    return vhfPointRotatedAroundCenter(refP, angle, cp);
+}
+
+/* 1996-06-??
+ * return the angle between the 3 points ccw
+ */
+float vhfAngleBetweenPoints(const NSPoint start, const NSPoint middle, const NSPoint end)
+{   double	dx, dy;
+    float	angle0, angle1;
+
+    /* calculate angle of start to 0 degree
+     */
+    dx = start.x - middle.x;
+    dy = start.y - middle.y;
+    if (!dx)
+        angle0 = (dy >= 0) ? 90.0 : 270;
+    else
+    {	angle0 = RadToDeg(atan(dy / dx));
+        if (dx<0)           /* 2nd, 3rd quadrant */
+            angle0 += 180;
+        if (dx>0 && dy<0)   /* 4th quadrant */
+            angle0 += 360;
+    }
+
+    /* calculate angle of end to 0 degree
+     */
+    dx = end.x - middle.x;
+    dy = end.y - middle.y;
+    if (!dx)
+        angle1 = (dy >= 0) ? 90.0 : 270;
+    else
+    {	angle1 = RadToDeg(atan(dy / dx));
+        if (dx<0)           /* 2nd, 3rd quadrant */
+            angle1 += 180;
+        if (dx>0 && dy<0)   /* 4th quadrant */
+            angle1 += 360;
+    }
+
+    if (angle1 < angle0)
+        angle1 += 360;
+    return angle1 - angle0;
+}
+
+
+/*
+ * Sorting
+ */
+
 /* created:   19.03.96
  * modified:  
  * purpose:   sort array of points with increasing distance to p0
@@ -475,20 +638,20 @@ void sortPointArray(NSPoint *pArray, int cnt, NSPoint p0)
 {   int		i;
 
     for (i=0; i<cnt-1; i++)
-    {	int	j, jMin;
-        float	lastDist, newDist;
-        NSPoint	p;
+    {	int     j, jMin;
+        float   lastDist, newDist;
+        NSPoint p;
 
         jMin = cnt;
-        lastDist=SqrDistPoints(pArray[i], p0);
+        lastDist = SqrDistPoints(pArray[i], p0);
         for (j=i+1; j<cnt; j++)
         {
-            if ((newDist=SqrDistPoints(pArray[j], p0)) < lastDist)
+            if ((newDist = SqrDistPoints(pArray[j], p0)) < lastDist)
             {	lastDist = newDist;
                 jMin = j;
             }
         }
-        if (jMin<cnt)
+        if (jMin < cnt)
         {   p = pArray[i];
             pArray[i] = pArray[jMin];
             pArray[jMin] = p;
@@ -552,8 +715,7 @@ NSRect vhfBoundsOfPoints(NSPoint *pts, int nPts)
  * purpose:   return YES, if point is in array
  *            use a tolerance for comparison!
  *            svIntersectCurveCurve needs this extremely large Tolerance!!
- * parameter: arc (pointer to arc)
- *            points (array for the start and end points [0] = start, [1] = end)
+ * parameter: 
  */
 BOOL pointInArray(NSPoint point, const NSPoint *array, int cnt)
 {   int	i;
@@ -573,11 +735,11 @@ BOOL pointWithToleranceInArray(NSPoint point, float tol, const NSPoint *array, i
     return NO;
 }
 
-/* created:  19.06.96
- * modified:
+/* created:  1996-06-19
+ * modified: 
  * purpose:  return YES, if value is in array
  */
-BOOL valueInArray(int val, int* array, int cnt)
+BOOL valueInArray(int val, int* array, int cnt) // DEPRECATED, use vhfIsIntValueInArray()
 {   int	i;
 
     for (i=0; i<cnt; i++)
@@ -585,14 +747,21 @@ BOOL valueInArray(int val, int* array, int cnt)
             return YES;
     return NO;
 }
+/* created:  1996-06-19
+ * modified: 2008-08-18
+ * purpose:  return YES, if value is in array
+ */
+BOOL vhfIsIntValueInArray(int val, int* array, int cnt)
+{   int	i;
+    
+    for (i=0; i<cnt; i++)
+        if ( array[i] == val )
+            return YES;
+    return NO;
+}
 
-/* created:  1993-07-08
- * modified: 2001-02-15
- * purpose:  return YES, if point is in array
- *           use a tolerance for comparison!
- *           svIntersectCurveCurve needs this extremely large Tolerance!!
- * parameter: arc (pointer to arc)
- *            points (array for the start and end points [0] = start, [1] = end)
+/* created:   1993-07-08
+ * modified:  2001-02-15
  */
 int removePointFromArray(NSPoint p, NSPoint *pArray, int cnt)
 {   int	i, j;
@@ -634,67 +803,10 @@ int vhfFilterPoints(NSPoint *pArray, int cnt, float tol)
     return cnt;
 }
 
-/* calculate the angle of p relative cp
+
+/*
+ * Bezier
  */
-float vhfAngleOfPointRelativeCenter(NSPoint p, NSPoint cp)
-{   float a, dx = p.x - cp.x, dy = p.y - cp.y;
-
-    if (!dx)
-        a = (dy >= 0) ? 90.0 : 270.0;
-    else
-    {	a = RadToDeg(atan(dy / dx));
-        if (dx<0) a += 180; /* 2, 3 */
-        if (dx>0 && dy<0) a += 360;	/* 4 */
-    }
-    return a;
-}
-
-/* calculates a point on the radius with a distance of angle (degree) from a reference point
- */
-NSPoint vhfPointAngleFromRefPoint(NSPoint cp, NSPoint refP, float angle)
-{
-    return vhfPointRotatedAroundCenter(refP, angle, cp);
-}
-
-/* 1996-06-??
- * return the angle between the 3 points ccw
- */
-float vhfAngleBetweenPoints(const NSPoint start, const NSPoint middle, const NSPoint end)
-{   double	dx, dy;
-    float	angle0, angle1;
-
-    /* calculate angle of start to 0 degree
-     */
-    dx = start.x - middle.x;
-    dy = start.y - middle.y;
-    if (!dx)
-        angle0 = (dy >= 0) ? 90.0 : 270;
-    else
-    {	angle0 = RadToDeg(atan(dy / dx));
-        if (dx<0)		/* 2nd, 3rd quadrant */
-            angle0 += 180;
-        if (dx>0 && dy<0)	/* 4th quadrant */
-            angle0 += 360;
-    }
-
-    /* calculate angle of end to 0 degree
-     */
-    dx = end.x - middle.x;
-    dy = end.y - middle.y;
-    if (!dx)
-        angle1 = (dy >= 0) ? 90.0 : 270;
-    else
-    {	angle1 = RadToDeg(atan(dy / dx));
-        if (dx<0)		/* 2nd, 3rd quadrant */
-            angle1 += 180;
-        if (dx>0 && dy<0)	/* 4th quadrant */
-            angle1 += 360;
-    }
-
-    if (angle1 < angle0)
-        angle1 += 360;
-    return angle1 - angle0;
-}
 
 /* created:   
  * modified:  
@@ -797,6 +909,165 @@ BOOL vhfGetGradientOfCurveAt(NSPoint *p, NSPoint pc[4], float t)
     return YES;
 }
 
+/* flatten line
+ *
+ * return: number of points in pts
+ *         pts has to be freed using NSZoneFree(NSDefaultMallocZone(), pts)
+ *
+ * modified: 2011-04-23
+ */
+int vhfConvertLineToLines(NSPoint pc[2], float flatness, NSPoint **pts)
+{   NSZone  *zone = NSDefaultMallocZone();
+    double	dx = pc[1].x-pc[0].x, dy = pc[1].y -pc[0].y;
+    double  length2 = dx*dx + dy*dy;
+    NSPoint *ptsPtr;
+
+    if ( flatness < 0.02 )
+        flatness = 0.02;
+
+    if (length2 > flatness * flatness)
+    {   NSZone  *zone = NSDefaultMallocZone();
+        int     i, n = Max(1, (int)ceil(sqrt(length2 / (flatness*flatness))));
+        NSPoint	len = (NSPoint){dx / (double)n, dy / (double)n};
+
+        *pts = NSZoneMalloc(zone, (n+1) * sizeof(NSPoint));
+        ptsPtr = *pts;
+
+        ptsPtr[0] = pc[0];
+
+        for (i=1; i<=n; i++)
+        {   ptsPtr[i].x = pc[0].x + (double)i * len.x;
+            ptsPtr[i].y = pc[0].y + (double)i * len.y;
+        }
+        return n+1;
+    }
+    *pts = NSZoneMalloc(zone, 2 * sizeof(NSPoint));
+    (*pts)[0] = pc[0];
+    (*pts)[1] = pc[1];
+    return 2;
+}
+
+/* flatten curve
+ *
+ * return: number of points in pts
+ *         pts has to be freed using NSZoneFree(NSDefaultMallocZone(), pts)
+ *
+ * modified: 2007-09-21
+ */
+int vhfConvertCurveToLines(NSPoint pc[4], float flatness, NSPoint **pts)
+{   NSPoint     *apPointList[3], *pPointRead, *pPointWrite, curvePoint;
+    double      aLength[3], ax, ay, bx, by, cx, cy, t, dist, maxdist;
+    int         i, i2, n = 1;
+    NSZone      *zone = NSDefaultMallocZone();
+
+    if ( flatness < 0.02 )
+        flatness = 0.02;
+    (apPointList[0]) = NSZoneMalloc(zone, (ldexp(3, 9)+1) * sizeof(NSPoint));
+    (apPointList[1]) = NSZoneMalloc(zone, (ldexp(3, 9)+1) * sizeof(NSPoint));
+    apPointList[2]   = 0;
+
+    * apPointList[0]    = pc[0];
+    *(apPointList[0]+1) = pc[1];
+    *(apPointList[0]+2) = pc[2];
+    *(apPointList[0]+3) = pc[3];
+
+    /* split curve in two curves until the wanted resolution has been achieved */
+    do
+    {   maxdist = dist = 0.0;
+
+        if ( n >= 9 && !(apPointList[1] = NSZoneRealloc(zone, apPointList[1], (ldexp(3,n)+1) * sizeof(NSPoint))) )
+        {   NSZoneFree(zone, apPointList[0]);
+            return 0;
+        }
+        pPointRead  = apPointList[0];
+        pPointWrite = apPointList[1];
+        *pPointWrite = *pPointRead;
+
+        /* scan curves in pPointRead */
+        for ( i2 = ldexp(1,n-1); i2 > 0; i2--)
+        {
+            *(pPointWrite+6) = *(pPointRead+3);
+            *(pPointWrite+1) = CenterPoint( * pPointRead,    *(pPointRead+1));
+            *(pPointWrite+3) = CenterPoint( *(pPointRead+1), *(pPointRead+2));
+            *(pPointWrite+5) = CenterPoint( *(pPointRead+2), *(pPointRead+3));
+            *(pPointWrite+2) = CenterPoint( *(pPointWrite+1), *(pPointWrite+3));
+            *(pPointWrite+4) = CenterPoint( *(pPointWrite+3), *(pPointWrite+5));
+            *(pPointWrite+3) = CenterPoint( *(pPointWrite+2), *(pPointWrite+4));
+
+            /* calculate the maximum deviation */
+            for ( i = 1; i <= 2; i++, pPointWrite += 3)
+            {
+                /* calculation of the curve with the functions
+                 * x(t) = ax*t^3 + bx*t^2 + cx*t + x0
+                 * y(t) = ay*t^3 + by*t^2 + cy*t + y0
+                 */
+                cx = 3*((pPointWrite+1)->x - pPointWrite->x);
+                bx = 3*((pPointWrite+2)->x - (pPointWrite+1)->x) - cx;
+                ax = (pPointWrite+3)->x - pPointWrite->x - bx - cx;
+                cy = 3*((pPointWrite+1)->y - pPointWrite->y);
+                by = 3*((pPointWrite+2)->y - (pPointWrite+1)->y) - cy;
+                ay = (pPointWrite+3)->y - pPointWrite->y - by - cy;
+                
+                /* calculation of the length of the single vector */
+                aLength[0] = sqrt(SqrDistPoints(pPointWrite[0], pPointWrite[1]));
+                aLength[1] = sqrt(SqrDistPoints(pPointWrite[1], pPointWrite[2]));
+                aLength[2] = sqrt(SqrDistPoints(pPointWrite[2], pPointWrite[3]));
+                
+                /* deviation for P1 */
+                t = aLength[0] + aLength[1] + aLength[2];
+                if (t == 0)
+                {   NSZoneFree(zone, apPointList[0]);
+                    NSZoneFree(zone, apPointList[1]);
+                    return 0;
+                }
+                t = aLength[0] / t;
+                curvePoint.x = ax*t*t*t + bx*t*t + cx*t + pPointWrite->x;
+                curvePoint.y = ay*t*t*t + by*t*t + cy*t + pPointWrite->y;
+                dist = SqrDistPoints(pPointWrite[1], curvePoint);
+                maxdist = (maxdist > dist) ? maxdist : dist;
+                
+                /* deviation for P2 */
+                t = aLength[0] + aLength[1] + aLength[2];
+                if (t == 0)
+                {   NSZoneFree(zone, apPointList[0]);
+                    NSZoneFree(zone, apPointList[1]);
+                    return 0;
+                }
+                t = (aLength[0] + aLength[1]) / t;
+                curvePoint.x = ax*t*t*t + bx*t*t + cx*t + pPointWrite->x;
+                curvePoint.y = ay*t*t*t + by*t*t + cy*t + pPointWrite->y;
+                dist = SqrDistPoints(pPointWrite[2], curvePoint);
+                maxdist = (maxdist > dist) ? maxdist : dist;
+            }
+            pPointRead  += 3;
+        }
+
+        apPointList[2] = apPointList[0];
+        apPointList[0] = apPointList[1];
+        apPointList[1] = apPointList[2];
+        n++;
+    } while ( maxdist > flatness*flatness);
+
+    NSZoneFree(zone, apPointList[1]);
+
+    *pts = apPointList[0];
+
+    /*{   NSPoint  pv0, pv1;
+
+        pPointRead = apPointList[0];
+        pv0 = *pPointRead++;
+        for (i = ldexp(3, n-1); i > 0; i--)
+        {   pv1 = *pPointRead++;
+            printf("%.1f %.1f ", pv1.x, pv1.y);
+            //[tmpStream appendLine:pv1.x :pv1.y];
+        }
+        printf("\n");
+        //free(apPointList[0]);
+    }*/
+
+    return ldexp(3, n-1) + 1;
+}
+
 BOOL vhfConvertCurveToArc(NSPoint pc[4], NSPoint *center, NSPoint *start, float *angle)
 {   NSPoint	pa, da, pb, db, ps[2];
     NSPoint	end = pc[3];
@@ -832,3 +1103,166 @@ BOOL vhfConvertCurveToArc(NSPoint pc[4], NSPoint *center, NSPoint *start, float
 
     return NO;
 }
+
+/* return rectangle fitting the start and end point of the arc and all quadrants in between
+ * created:  2004-08-04
+ * modified: 2004-08-04
+ */
+NSRect vhfBoundsOfArc(NSPoint center, float radius, float begAngle, float angle)
+{   NSRect	bounds;
+    float	ba, ea;
+    NSPoint	p, ll, ur, start, end;
+
+    if (Abs(angle) >= 360.0)
+    {
+        bounds.origin.x = center.x - radius;
+        bounds.origin.y = center.y - radius;
+        bounds.size.width = bounds.size.height = 2.0 * radius;
+        return bounds;
+    }
+//printf("center = (%f %f) radius = %f begAngle = %f angle %f\n", center.x, center.y, radius, begAngle, angle);
+
+    start = vhfPointRotatedAroundCenter(NSMakePoint(center.x+radius, center.y), begAngle, center);
+    end   = vhfPointRotatedAroundCenter(start, angle, center);
+//printf("start = (%f %f)  end = (%f %f)\n", start.x, start.y, end.x, end.y);
+
+    ll.x = Min(start.x, end.x); ll.y = Min(start.y, end.y);
+    ur.x = Max(start.x, end.x); ur.y = Max(start.y, end.y);
+
+    /* we need positive angles with ba < ea */
+    ba = (angle>=0.0) ? begAngle : (begAngle+angle);
+    if (ba < 0.0)   ba += 360.0;
+    if (ba > 360.0) ba -= 360.0;
+    ea = ba + Abs(angle);
+//printf("ba = %f ea = %f\n", ba, ea);
+
+    if ((ba < 360.0 || ba >= ea-360.0-TOLERANCE) && ea >= 360.0)	// 0 degree
+    {   p.x = center.x+radius; p.y = center.y;
+        ll.x = Min(ll.x, p.x); ll.y = Min(ll.y, p.y);
+        ur.x = Max(ur.x, p.x); ur.y = Max(ur.y, p.y);
+    }
+    if ((ba <= 90.0 && ea >= 90.0) || (ba<=450 && ea>=450))		// 90 degree
+    {   p.x = center.x; p.y = center.y+radius;
+        ll.x = Min(ll.x, p.x); ll.y = Min(ll.y, p.y);
+        ur.x = Max(ur.x, p.x); ur.y = Max(ur.y, p.y);
+    }
+    if ((ba <= 180.0 && ea >= 180.0) || (ba<=540 && ea>=540))		// 180 degree
+    {   p.x = center.x-radius; p.y = center.y;
+        ll.x = Min(ll.x, p.x); ll.y = Min(ll.y, p.y);
+        ur.x = Max(ur.x, p.x); ur.y = Max(ur.y, p.y);
+    }
+    if ((ba <= 270.0 && ea >= 270.0) || (ba<=630 && ea>=630))		// 270 degree
+    {   p.x = center.x; p.y = center.y-radius;
+        ll.x = Min(ll.x, p.x); ll.y = Min(ll.y, p.y);
+        ur.x = Max(ur.x, p.x); ur.y = Max(ur.y, p.y);
+    }
+
+    bounds.origin = ll;
+    bounds.size.width  = MAX(ur.x - ll.x, 0.001); // 1.0
+    bounds.size.height = MAX(ur.y - ll.y, 0.001);
+
+    return bounds;
+}
+
+/*
+ * modified:  01.08.93 30.04.96
+ * Author:    Martin Dietterle
+ * purpose:   get bounds of curve
+ * parameter: curve points, bounds
+ * return:    none
+ */
+NSRect vhfBoundsOfCurve( NSPoint pc[4] )
+{   double	max, min, ft;
+    double	ax, bx, cx, ay, by, cy, t, ts[2];
+    int		i, cnt;
+    NSRect	bounds;
+
+    /* Berechnung und Festlegung der Bezierkurve durch die Gleichungen
+     *  x(t) = ax*t^3 + bx*t^2 + cx*t + x(0)
+     *  y(t) = ay*t^3 + by*t^2 + cy*t + y(0)
+     */
+    cx = 3.0*(pc[1].x - pc[0].x);
+    bx = 3.0*(pc[2].x - pc[1].x) - cx;
+    ax = pc[3].x - pc[0].x - bx - cx;
+    cy = 3.0*(pc[1].y - pc[0].y);
+    by = 3.0*(pc[2].y - pc[1].y) - cy;
+    ay = pc[3].y - pc[0].y - by - cy;
+
+    /* Ermittlung des maximalen und minimalen X-Wertes der Bezier-Kurve. */
+    min = Min(pc[0].x, pc[3].x);
+    max = Max(pc[0].x, pc[3].x);
+
+    cnt = svPolynomial2(3.0 * ax, 2.0 * bx, cx, ts);
+    for (i=0; i<cnt; i++)
+    {
+        t = ts[i];
+        if ( t>0.0 && t<1.0)
+        {
+            ft = ax*t*t*t + bx*t*t + cx*t + pc[0].x;
+            if (min > ft)
+                min = ft;
+            else if (max < ft)
+                max = ft;
+        }
+    }
+
+    bounds.origin.x = min;
+    bounds.size.width = max - min;
+
+    /* Ermittlung des maximalen und minimalen Y-Wertes der Bezier-Kurve. */
+    min = Min(pc[0].y, pc[3].y);
+    max = Max(pc[0].y, pc[3].y);
+
+    cnt = svPolynomial2(3.0 * ay, 2.0 * by, cy, ts);
+    for (i=0; i<cnt; i++)
+    {
+        t = ts[i];
+        if ( t>0.0 && t<1.0)
+        {
+            ft = ay*t*t*t + by*t*t + cy*t + pc[0].y;
+            if (min > ft)
+                min = ft;
+            else if (max < ft)
+                max = ft;
+        }
+    }
+
+    bounds.origin.y = min;
+    bounds.size.height = max - min;
+    return bounds;
+}
+
+/* flatten arc into lines
+ * return: number of points in pts
+ *         pts has to be freed using NSZoneFree(NSDefaultMallocZone(), pts)
+ *
+ * modified: 2007-09-22
+ */
+int vhfConvertArcToLines(NSPoint center, float radius, float begAngle, float angle, float flatness, NSPoint **pts)
+{   double	chordAngle, dx = radius - flatness;  // we tune the flatness to make it smoother
+    int         n, i, nPts = 0;
+    NSPoint     startPoint, p, *ptsPtr;
+    NSZone      *zone = NSDefaultMallocZone();
+
+    startPoint = vhfPointRotatedAroundCenter(NSMakePoint(center.x+radius, center.y), begAngle, center);
+
+    if (dx <= 0.0)
+        n = 1;
+    else
+    {   chordAngle = RadToDeg(acos(dx/radius))*2.0;
+        chordAngle = Limit(chordAngle, 0.1, 5.0);
+        n = (int)ceil(Abs(angle) / chordAngle);         // number of lines
+    }
+
+    *pts = NSZoneMalloc(zone, (n+1) * sizeof(NSPoint));
+    ptsPtr = *pts;
+
+    chordAngle = angle / n;				// chord angle to get lines of equal length
+    for ( i=1; i<=n; i++ )
+    {   float	a = (double)i * chordAngle;
+
+        p = vhfPointRotatedAroundCenter(startPoint, a, center);
+        ptsPtr[nPts++] = p;
+    }
+    return nPts;
+}
